; Fargo IDE header
;=============================================================================
; Last save: Sat Jan 23 15:38:59  1999
; 9.15 143.0 143.0 143.0 143.0 143.0
;=============================================================================
;----------------------------------------------------------------------------------------------
; This is the source of a Fargo program that is intended to be executed from the TIOS
; command line. It returns the name of all files within a folder as a list of strings.
;
;; Copyright (c) 1999 David Kuehling

;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to
;; deal in the Software without restriction, including without limitation the
;; rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
;; sell copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice (including the next
;; paragraph) shall be included in all copies or substantial portions of the
;; Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
;; FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
;; IN THE SOFTWARE.

; Last Modification: Jan 22 1999
;
; Thanks to:
;	Gareth James for files.txt 
;	David Ellsworth for Fargo and the testfunc.asm example program
;----------------------------------------------------------------------------------------------


;----------------------------------------------------------------------------------------------
		INCLUDE	"tios.h"
;----------------------------------------------------------------------------------------------

;----------------------------------------------------------------------------------------------
		XDEF	_main
		XDEF	_tibasic
;----------------------------------------------------------------------------------------------

sizeof_SYM_ENTRY	EQU tios::SYM_ENTRY.hVal+2
;----------------------------------------------------------------------------------------------
; Return the address in a2, that belongs to the handle, specified by d0.
; d0 is destroyed.
;----------------------------------------------------------------------------------------------
DEREF_d0_a2:	tios::DEREF	d0,a2
		rts

;----------------------------------------------------------------------------------------------
; Push the token, specified by d0.b onto the estack.
;----------------------------------------------------------------------------------------------
PushToken:	move.w	d0,-(a7)
		jsr	tios::push_quantum
		addq.w	#2,a7
		rts

;----------------------------------------------------------------------------------------------
; MAIN
;----------------------------------------------------------------------------------------------
_main:		move.w	#$E5,d0			; mark end of list ('}') (return value)
		bsr	PushToken

		; get the address of the argument-string ("s")
		pea	ArgName(PC)		; get address of argument's entry in temp folder
		move.w	tios::DefTempHandle,-(a7)		
		jsr	tios::FindSymEntry	; a0.l := pointer to symbol entry of argument
		addq.l	#6,a7
		move.w	tios::SYM_ENTRY.hVal(a0),d0	; d0 := handle of argument
		bsr	DEREF_d0_a2			; a2 := pointer to argument

		; check, whether argument is a string
		move.w	(a2),d0			; d0 = length of variable
		cmpi.b	#$2D,1(a2,d0)		; verify STR tag (at end of variable)
		beq	\ArgStrOk
		move.w	#130,d0			; if it isn't a STR: display error and exit
		bra	\Error
\ArgStrOk:
		; search for the folder whose name is given by the argument string ("s")
		pea	3(a2)			; push pointer to folder's name (the argument)
		move.w	FOLDER_LIST_HANDLE,-(a7)
		jsr	tios::FindSymEntry
		addq.l	#6,a7
		move.l	a0,d0			; check, whether a0.l is zero
		bne	\ArgFoldFound		; if it isn't zero: folder wasn't found
		move.w	#420,d0			; -> display error message and exit
		bra	\Error
\ArgFoldFound:
		; get address of file table of that folder
		move.w	tios::SYM_ENTRY.hVal(a0),d0	; d0 := handle of file table
		bsr	DEREF_d0_a2			; a2 := pointer to file table
		addq.l	#2,a2
		move.w	(a2)+,d3		; d3 := number of files

		; since elements of lists are stored in reversed order, we have to
		; process the file table from its end
		move.w	d3,d4			
		mulu	#sizeof_SYM_ENTRY,d4	; d4 := size of file table in bytes
		lea	-12(a2,d4),a2		; a2 := pointer to last file in file table

		; now we push all file names as strings onto the estack
 		; everything is done in reversed order, that's why I first push the string end,
		; then the string itself and then the string begin (#0 and STR tag)
		; the same applys to the LIST, I first push the end-mark ('}') and then
		; the beginning mark and LIST tag ('{')
		subq.w	#1,d3			; d3 := counter  (#folders - 1)
\Loop:		lea	File(PC),a3		; copy file name into zero-terminated string
		move.l	(a2)+,(a3)		; a3 := pointer to zero-terminated name
		move.l	(a2),4(a3)		; a2 := pointer to file name + 4

		moveq	#0,d0			; mark end of string
		bsr	PushToken

		; copy folder-name onto estack (including the trailing zero)
\__StrLoop:	move.b	(a3)+,d0		; push one character onto estack
		bsr	PushToken
		tst.b	-1(a3)
		bne	\__StrLoop
		moveq	#$2D,d0			; push STR tag (beginning of string)
		bsr	PushToken

		lea	-sizeof_SYM_ENTRY-4(a2),a2	; process next file name
		dbra	d3,\Loop

\End:		move.w	#$D9,d0			; push a LIST - tag (beginning of list; ('{'))
		bsr	PushToken
		move.b	#2,tios::globals+$126E	; indicate that there is a return value

		rts


		; Display the error message that correspond to the errno in d0 and exit
\Error:		move.w	d0,-(a7)
		jsr	tios::ERD_dialog
		addq.l	#2,a7
		bra	\End
		

;==============================================================================================
; Data
;==============================================================================================
		EVEN
File:		dc.b	"        ",0		; to zero-terminate a file name
ArgName:	dc.b 	"s",0			; the name of the argument (the folder name)


;==============================================================================================
; Function Definition
;==============================================================================================
		SECTION _tibasic
		dc.b	$E9		
		dc.b	$0F,$E4			; EndFunc
		dc.b	$00,$E8			; :
		dc.b	$17,$E4			; Func
		dc.b	$E5,$03			; (s)
		dc.b	$00,$00,$40,$DC 
_tibasic:
		END






